package main

import (
	"fmt"
	"io/ioutil"
	"sort"

	"github.com/pkg/errors"
	"gopkg.in/yaml.v2"
)

const (
	AuthorAccessRead  = 0
	AuthorAccessCheck = 1
	AuthorAccessWrite = 2
)

const DefaultConfigPath = "~/.config/stick/stick.yml"

type Config struct {
	Salt  string
	Serve string

	Authors   []*AuthorConfig
	Notebooks []*NotebookConfig

	Debug bool
}

type AuthorConfig struct {
	Email string
	Name  string
}

type NotebookConfig struct {
	Label string
	Repo  string
	Serve map[string]int
}

func ReadConfigFile(path string) (*Config, error) {
	var err error
	if path == "" {
		path = DefaultConfigPath
	}
	path, err = expandPath(path)
	if err != nil {
		return nil, err
	}

	data, err := ioutil.ReadFile(path)
	if err != nil {
		return nil, err
	}

	c := &Config{}
	err = c.load(data)
	return c, err
}

func ReadConfigData(data []byte) (*Config, error) {
	c := &Config{}
	err := c.load(data)
	return c, err
}

func (c *Config) load(data []byte) error {
	var config map[string]interface{}
	err := yaml.Unmarshal(data, &config)
	if err != nil {
		return err
	}

	if salt, ok := config["salt"]; ok {
		c.Salt = salt.(string)
	}

	if serve, ok := config["serve"]; ok {
		c.Serve = serve.(string)
	}

	if as, ok := config["authors"]; ok {
		auths := as.(map[interface{}]interface{})
		for aemail, aname := range auths {
			c.Authors = append(c.Authors, &AuthorConfig{Email: aemail.(string), Name: aname.(string)})
		}
	}

	if nbsv, ok := config["notebooks"]; ok {
		nbs := nbsv.(map[interface{}]interface{})
		var labels []string
		for label := range nbs {
			labels = append(labels, label.(string))
		}
		sort.Strings(labels)
		for _, label := range labels {
			nbconfig := &NotebookConfig{Label: label, Serve: make(map[string]int)}

			nbo := nbs[label].(map[interface{}]interface{})
			for option, v := range nbo {
				switch option.(string) {
				case "repo":
					nbconfig.Repo = v.(string)
				case "serve":
					nbsv := v.([]interface{})
					for _, nbsvi := range nbsv {
						switch nbsvi.(type) {
						case string:
							nbconfig.Serve[nbsvi.(string)] = AuthorAccessWrite
						case map[interface{}]interface{}:
							nbsvx := nbsvi.(map[interface{}]interface{})
							for nbsvxi, nbsvxa := range nbsvx {
								access := -1
								switch nbsvxa.(string) {
								case "read":
									access = AuthorAccessRead
								case "check":
									access = AuthorAccessCheck
								case "write":
									access = AuthorAccessWrite
								}
								if access == -1 {
									return errors.New(fmt.Sprintf("invalid serve configuration: invalid access level specified for %s (%s should be read/check/write)", nbsvxi, nbsvxa.(string)))
								}

								nbconfig.Serve[nbsvxi.(string)] = access
							}
						default:
							return errors.New(fmt.Sprintf("invalid serve configuration: %T", nbsvi))
						}
					}
				}
			}

			if nbconfig.Repo == "" {
				return errors.New("invalid notebook configuration: no repo path supplied")
			}

			c.Notebooks = append(c.Notebooks, nbconfig)
		}
	}

	return nil
}
