package main

import (
	"log"
	"sync"

	"github.com/pkg/errors"
)

type Stick struct {
	Config *Config

	Authors     map[string]*Author
	AuthorsLock sync.RWMutex

	Notebooks     map[string]*Notebook
	NotebooksLock sync.RWMutex
}

func Initialize(configPath string, debug bool) {
	var err error

	stick = &Stick{}
	stick.Config, err = ReadConfigFile(configPath)
	CheckError(err)

	stick.Config.Debug = debug
	StickSalt = []byte(stick.Config.Salt)

	stick.loadAuthors()
	stick.loadNotebooks()
}

func (s *Stick) loadAuthors() {
	// TODO: Allow reloading config

	stick.Authors = map[string]*Author{}
	for _, author := range stick.Config.Authors {
		stick.Authors[hash(author.Email)] = &Author{Key: hash(author.Email), Email: author.Email, Name: author.Name}
	}
}

func (s *Stick) loadNotebooks() {
	// TODO: Allow reloading config

	stick.Notebooks = make(map[string]*Notebook)
	for _, nbconfig := range stick.Config.Notebooks {
		notebook, err := LoadNotebook(nbconfig.Repo)
		if err != nil {
			log.Fatal(errors.Wrapf(err, "failed to load %s", nbconfig.Repo))
		}

		notebook.ID = hash(nbconfig.Label)
		notebook.Label = nbconfig.Label

		notebook.Serve = make(map[string]int)
		for serveauthor, serveaccess := range nbconfig.Serve {
			notebook.Serve[hash(serveauthor)] = serveaccess
		}

		stick.Notebooks[hash(nbconfig.Label)] = notebook
	}

	var author *Author
	for _, notebook := range stick.Notebooks {
		for serveauth := range notebook.Serve {
			author = stick.getAuthor(serveauth)
			if author == nil {
				continue
			}

			author.Notebooks = append(author.Notebooks, notebook)
		}
	}

}

func (s *Stick) getAuthor(key string) *Author {
	s.AuthorsLock.RLock()
	defer s.AuthorsLock.RUnlock()

	if a, ok := s.Authors[key]; ok {
		return a
	}

	return nil
}

func (s *Stick) getNotebook(key string) *Notebook {
	s.NotebooksLock.RLock()
	defer s.NotebooksLock.RUnlock()

	if n, ok := s.Notebooks[key]; ok {
		return n
	}

	return nil
}
